unit Sample14;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Blocks, Statsim, ComCtrls;

type
  TForm1 = class(TForm)
    Memo: TMemo;
    Button1: TButton;
    MultiRand: TMultiRand;
    Label1: TLabel;
    ProgressBar: TProgressBar;
    Tabulator0: TTabulator;
    Label2: TLabel;
    EditNum: TEdit;
    Tabulator1: TTabulator;
    RegularPatients: TGenerator;
    ToothpainPacients: TGenerator;
    Doctors: TStorage;
    ExitDoor: TTerminator;
    Model: TModel;
    Hall: TQueuePrty;
    procedure Button1Click(Sender: TObject);
    procedure EditNumExit(Sender: TObject);
    procedure RegularPatientsBeforeTimeGoOn(Sender: TAggregate);
    procedure RegularPatientsExit(Sender: TBlock; Trans: TTransaction);
    procedure RegularPatientsRouting(Sender: TBlock; Trans: TTransaction);
    procedure ToothpainPacientsAfterGeneration(Sender: TBlock;
      Trans: TTransaction);
    procedure ToothpainPacientsBeforeTimeGoOn(Sender: TAggregate);
    procedure ToothpainPacientsExit(Sender: TBlock; Trans: TTransaction);
    procedure ToothpainPacientsRouting(Sender: TBlock;
      Trans: TTransaction);
    procedure DoctorsEnter(Sender: TBlock; Trans: TTransaction);
    procedure DoctorsRouting(Sender: TBlock; Trans: TTransaction);
    procedure ExitDoorEnter(Sender: TBlock; Trans: TTransaction);
    procedure ModelNewTime(Sender: TAggregate; Trans: TTransaction);
    procedure HallRouting(Sender: TBlock; Trans: TTransaction);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

  MyTransaction = class(TTransaction)
  public
    BirthTime: real;
  end;

var
  Form1: TForm1;
  LimitTime: real;           // Simulation time
  QCapacity: longint;        // Queue capacity

implementation

{$R *.DFM}

procedure TForm1.Button1Click(Sender: TObject);
begin
   LimitTime:=720.0;
   Memo.Lines.Clear;
   EditNumExit(Sender);
   Model.SetTransactionClass(MyTransaction);
   Model.Simulate(LimitTime);
   Memo.Lines.Add('1. Regular patients');
   Memo.Lines.Add('   Number of patients: '+IntToStr(Tabulator0.Count));
   Memo.Lines.Add('   The average time spent in clinic: '+FormatFloat('0.00',
                                                          Tabulator0.Mean));

   Memo.Lines.Add('   The deviation of time spent in clinic: '+FormatFloat('0.00',
                                                          Tabulator0.Deviation));
   Memo.Lines.Add('2. Patients with tooth pain');
   Memo.Lines.Add('   Number of patients: '+IntToStr(Tabulator1.Count));
   Memo.Lines.Add('   The average time spent in clinic: '+FormatFloat('0.00',
                                                          Tabulator1.Mean));

   Memo.Lines.Add('   The deviation of time spent in clinic: '+FormatFloat('0.00',
                                                          Tabulator1.Deviation));
   Model.Reset;
end;


procedure TForm1.EditNumExit(Sender: TObject);
begin
   try
      QCapacity:=StrToInt(Trim(EditNum.Text));
   except
      MessageDlg('Invalid number',mtError,[mbOK],0);
      EditNum.SetFocus;
      Exit;
   end;
   if QCapacity<0 then
   begin
      MessageDlg('Invalid number',mtError,[mbOK],0);
      EditNum.SetFocus;
      Exit;
   end;
   Doctors.Capacity:=QCapacity;
end;


procedure TForm1.RegularPatientsBeforeTimeGoOn(Sender: TAggregate);
begin
  Sender.NextTime(0.0);
end;

procedure TForm1.RegularPatientsExit(Sender: TBlock; Trans: TTransaction);
begin
  (Trans as MyTransaction).BirthTime:=ModelTime;
  Sender.NextTime(MultiRand.Exponential(4.0));
end;

procedure TForm1.RegularPatientsRouting(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(Hall);
end;

procedure TForm1.ToothpainPacientsAfterGeneration(Sender: TBlock;
  Trans: TTransaction);
begin
  Trans.SetPrty(1);
end;

procedure TForm1.ToothpainPacientsBeforeTimeGoOn(Sender: TAggregate);
begin
  Sender.NextTime(0.0);
end;

procedure TForm1.ToothpainPacientsExit(Sender: TBlock;
  Trans: TTransaction);
begin
   (Trans as MyTransaction).BirthTime:=ModelTime;
   Sender.NextTime(MultiRand.Exponential(25.0));
end;

procedure TForm1.ToothpainPacientsRouting(Sender: TBlock;
  Trans: TTransaction);
begin
  Sender.PassTo(Hall);
end;

procedure TForm1.DoctorsEnter(Sender: TBlock; Trans: TTransaction);
begin
   Sender.NextTime(MultiRand.Uniform(7.0,15.0));
end;

procedure TForm1.DoctorsRouting(Sender: TBlock; Trans: TTransaction);
begin
   Sender.PassTo(ExitDoor);
end;

procedure TForm1.ExitDoorEnter(Sender: TBlock; Trans: TTransaction);
begin
   if Trans.GetPrty=0 then
      Tabulator0.PutValue(ModelTime-(Trans as MyTransaction).BirthTime)
   else
      Tabulator1.PutValue(ModelTime-(Trans as MyTransaction).BirthTime);
end;
    
procedure TForm1.ModelNewTime(Sender: TAggregate; Trans: TTransaction);
var  NewPos: integer;
     Ratio: real;
begin
   Ratio:=ModelTime/LimitTime;
   if Ratio>1.0 then Ratio:=1.0;
   NewPos:=Round(Ratio*100.0);
   with ProgressBar do
      if Position<>NewPos then Position:=NewPos;
end;

procedure TForm1.HallRouting(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(Doctors);
end;

end.
