unit Sample9;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Blocks, Statsim;

const
  ChangeTime: array[1..5] of real = (0.0,120.0,240.0,420.0,540.0);
  ArrivalMeans: array[1..5] of real = (14.0,12.0,10.0,12.0,14.0);

type
  TForm1 = class(TForm)
    Memo: TMemo;
    Button1: TButton;
    MultiRand: TMultiRand;
    Label1: TLabel;
    Label2: TLabel;
    EditNum: TEdit;
    Entrance: TGenerator;
    Hall: TQueue;
    Barber: TServer;
    ExitDoor: TTerminator;
    PeopleInStreet: TCreator;
    Model: TModel;
    procedure Button1Click(Sender: TObject);
    procedure EditNumExit(Sender: TObject);
    procedure EntranceBeforeTimeGoOn(Sender: TAggregate);
    procedure EntranceExit(Sender: TBlock; Trans: TTransaction);
    procedure EntranceRouting(Sender: TBlock; Trans: TTransaction);
    procedure HallRouting(Sender: TBlock; Trans: TTransaction);
    procedure BarberEnter(Sender: TBlock; Trans: TTransaction);
    procedure BarberRouting(Sender: TBlock; Trans: TTransaction);
    procedure PeopleInStreetRouting(Sender: TBlock; Trans: TTransaction);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  NumberOfPeople: integer;

implementation

{$R *.DFM}

procedure TForm1.Button1Click(Sender: TObject);
begin
   try
      NumberOfPeople:=StrToInt(Trim(EditNum.Text));
   except
      MessageDlg('Invalid value',mtError,[mbOK],0);
      EditNum.SetFocus;
      Exit;
   end;
   if NumberOfPeople<0 then
   begin
      MessageDlg('Invalid value',mtError,[mbOK],0);
      EditNum.SetFocus;
      Exit;
   end;

   Memo.Lines.Clear;
   Model.Simulate(720.0);
   Memo.Lines.Add('Usage of the barber: '+FormatFloat('0.00',Barber.Usage));
   Memo.Lines.Add('Average queue length: '+FormatFloat('0.00',Hall.AverageCount));
   Memo.Lines.Add('Maximal queue length: '+IntToStr(Hall.MaxCount));
   Memo.Lines.Add('Average waiting time: '+FormatFloat('0.00',Hall.AverageTime));
   Memo.Lines.Add('Deviation of waiting time: '+FormatFloat('0.00',Hall.DeviationTime));
   Memo.Lines.Add('Average waiting time / excluding zero times: '+FormatFloat('0.00',Hall.SAverageTime));
   Memo.Lines.Add('Deviation of waiting time/ excluding zero times: '+FormatFloat('0.00',Hall.SDeviationTime));
   MultiRand.Reset;
   Model.Reset;
end;



procedure TForm1.EditNumExit(Sender: TObject);
begin
   try
      NumberOfPeople:=StrToInt(Trim(EditNum.Text));
   except
      MessageDlg('Invalid value',mtError,[mbOK],0);
      EditNum.SetFocus;
      Exit;
   end;
   if NumberOfPeople<0 then
   begin
      MessageDlg('Invalid value',mtError,[mbOK],0);
      EditNum.SetFocus;
      Exit;
   end;
end;


procedure TForm1.EntranceBeforeTimeGoOn(Sender: TAggregate);
begin
  Sender.NextTime(MultiRand.Exponential(11.0));
  if NumberOfPeople>0 then
     PeopleInStreet.Generate(NumberOfPeople); // Direct generation of transactions
end;

procedure TForm1.EntranceExit(Sender: TBlock; Trans: TTransaction);
begin
  Sender.NextTime(MultiRand.Exponential(11.0));
end;

procedure TForm1.EntranceRouting(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(Hall);
end;

procedure TForm1.HallRouting(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(Barber);
end;


procedure TForm1.BarberEnter(Sender: TBlock; Trans: TTransaction);
begin
  Sender.NextTime(MultiRand.Exponential(10.0));
end;

procedure TForm1.BarberRouting(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(ExitDoor);
end;

procedure TForm1.PeopleInStreetRouting(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(Hall);
end;

end.
